//package ???;

import java.util.Arrays;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Set;

/**
 * Hilfs-Methoden zum Umgang mit List's und Map's.
 *
 * @author Heiner Kcker
 */
public class CollectionUtil
{

  /**
   * Constructor private, dont instantiatate this class.
   */
  private CollectionUtil()
  {
  }

  /** Schalter fr streng (Exception wenn Wert nicht gefunden) oder nicht (return null) */
  public static final boolean STRICT = false;


  /**
   * Abfrage einer hierarchischen Collection aus
   * Map's und List's mit einer beliebigen Anzahl
   * als Pfad verknpfter Key-Objekte.
   *
   * @param pCollection
   * @param pKeyPathArr
   * @return
   */
  public static Object path( //--
      final Object pCollection, //--
      final Object... pKeyPathArr)
  {
    Object tVal = pCollection;
    for (Object keyRunObj : pKeyPathArr)
    {
      if ( tVal == null )
      {
        if (STRICT)
        {
          // bei scharfer (intoleranter) Variante eine Exception werfen
          throw new RuntimeException( "no value found for param [" + keyRunObj + "] from path " + Arrays.asList( pKeyPathArr ) );
        }
        return null;
      }
      if ( tVal instanceof Map )
      {
        Map map = (Map) tVal;
        tVal = map.get( keyRunObj );
      }
      else if ( tVal instanceof List //--
          && keyRunObj instanceof Integer )
      {
        List list = (List) tVal;
        tVal = list.get( (Integer) keyRunObj );
      }
      else
      {
        if (STRICT)
        {
          // bei scharfer (intoleranter) Variante eine Exception werfen
          throw new RuntimeException( "no value found for param [" + keyRunObj + "] from path " + Arrays.asList( pKeyPathArr ) );
        }
        return null;
      }
    }
    return tVal;
  }


  /**
   * Ermittlung des ersten Key-Objektes der
   * bergebenen Map.
   *
   * @param pMap
   * @return das erste Key-Objekt
   */
  public static final Object getFirstKey(//--
      final Map pMap)
  {
    final Set keySet = pMap.keySet();
    for (Iterator iter = keySet.iterator(); iter.hasNext();)
    {
      return iter.next();
    }
    return null;
  }

  /** Einrckung fr die formatierte Auagabe von List' oder Map's */
  public static final String INDENT = "  ";

  /** Zeilenumbruch fr die formatierte Auagabe von List' oder Map's */
  public static final char NL = '\n';

  /** Schlssel Wert Trenner fr die formatierte Auagabe von List' oder Map's */
  public static final String KEY_VALUE_SEPERATOR = " => ";


  /**
   * Formatierte Ausgabe der bergebenen List.
   *
   * @param pList
   * @return
   */
  public static final String toStr( //--
      final List pList)
  {
    if ( pList == null )
    {
      return null;
    }
    return toStr( pList, "" );
  }

  /**
   * Formatierte Ausgabe der bergebenen Map.
   *
   * @param pMap
   * @return
   */
  public static final String toStr( //--
      final Map pMap)
  {
    if ( pMap == null )
    {
      return null;
    }
    return toStr( pMap, "" );
  }


  /**
   * Rekursive Methode zur formatierten
   * Ausgabe der bergebenen List.
   *
   * @param pList
   * @param pIndent
   * @return
   */
  private static String toStr(//--
      final List pList, //--
      final String pIndent)
  {
    int i = 0;
    final StringBuffer strBuff = new StringBuffer(1024);
    for (Object obj : pList)
    {
      strBuff.append( NL );
      strBuff.append( pIndent );
      strBuff.append( "[" );
      strBuff.append( i++ );
      strBuff.append( "]" );

      if ( obj instanceof List )
      {
        strBuff.append( NL );
        strBuff.append( pIndent );
        strBuff.append( "[" );
        strBuff.append( toStr( (List) obj, pIndent + INDENT ) );
        strBuff.append( NL );
        strBuff.append( pIndent );
        strBuff.append( "]" );
      }
      else if ( obj instanceof Map )
      {
        strBuff.append( NL );
        strBuff.append( pIndent );
        strBuff.append( "{" );
        strBuff.append( toStr( (Map) obj, pIndent + INDENT ) );
        strBuff.append( NL );
        strBuff.append( pIndent );
        strBuff.append( "}" );
      }
      else
      {
        strBuff.append( KEY_VALUE_SEPERATOR );
        strBuff.append( obj );
      }
    }
    return strBuff.toString();
  }


  /**
   * Rekursive Methode zur formatierten
   * Ausgabe der bergebenen Map.
   *
   * @param pMap
   * @param pIndent
   * @return
   */
  private static String toStr(//--
      final Map pMap, //--
      final String pIndent)
  {
    final StringBuffer strBuff = new StringBuffer(1024);
    final Set<Map.Entry> entrySet = pMap.entrySet();
    for (Map.Entry entry : entrySet)
    {
      strBuff.append( NL );
      strBuff.append( pIndent );
      strBuff.append( "(" );
      strBuff.append( entry.getKey() );
      strBuff.append( ")" );

      final Object value = entry.getValue();

      if ( value instanceof List )
      {
        strBuff.append( NL );
        strBuff.append( pIndent );
        strBuff.append( "[" );
        strBuff.append( toStr( (List) value, pIndent + INDENT ) );
        strBuff.append( NL );
        strBuff.append( pIndent );
        strBuff.append( "]" );
      }
      else if ( value instanceof Map )
      {
        strBuff.append( NL );
        strBuff.append( pIndent );
        strBuff.append( "{" );
        strBuff.append( toStr( (Map) value, pIndent + INDENT ) );
        strBuff.append( NL );
        strBuff.append( pIndent );
        strBuff.append( "}" );
      }
      else
      {
        strBuff.append( KEY_VALUE_SEPERATOR );
        strBuff.append( value );
      }
    }
    return strBuff.toString();
  }

}


